module Deque where

import Test.QuickCheck

data Deque a = Deque
  { front :: [a]
  , back  :: [a]
  } deriving (Show, Eq)

empty :: Deque a
empty = Deque [] []

addFirst :: a -> Deque a -> Deque a
addFirst x q = q { front = x : front q }

addLast :: a -> Deque a -> Deque a
addLast x q = q { back = x : back q }

removeFirst :: Deque a -> Maybe (a, Deque a)
removeFirst q = case q of
  Deque [] []     -> Nothing
  Deque (x:xs) ys -> Just (x, Deque xs ys)
  Deque [] ys     -> let (as, bs) = splitInHalf ys 
                     in removeFirst (Deque (reverse bs) as)

removeLast :: Deque a -> Maybe (a, Deque a)
removeLast q = case q of
  Deque [] []     -> Nothing
  Deque xs (y:ys) -> Just (y, Deque xs ys)
  Deque xs []     -> let (as, bs) = splitInHalf xs
                     in removeLast (Deque as (reverse bs))

splitInHalf :: [a] -> ([a], [a])
splitInHalf xs = splitAt (length xs `div` 2) xs

null :: Deque a -> Bool
null q = case q of
  Deque [] [] -> True
  _           -> False

toList :: Deque a -> [a]
toList q = front q ++ reverse (back q)
